############################################################
# wxPyCalc-refactored-v3.py
#
# Created 11/11/2008 by Mike Driscoll
#
# Description: A basic calculator
############################################################

import wx

class MainPanel(wx.Panel):
    def __init__(self, parent):
        wx.Panel.__init__(self, parent=parent, id=wx.ID_ANY)
        self.parent = parent
        self.formula = []
        self.currentVal = "0"
        self.previousVal = "0"
        self.operator = None
        self.operatorFlag = False
        self.createDisplay()
        
    def createDisplay(self):
        """
        Create the calculator display
        """
        mainSizer = wx.BoxSizer(wx.VERTICAL)
        masterBtnSizer = wx.BoxSizer(wx.HORIZONTAL)
        vBtnSizer = wx.BoxSizer(wx.VERTICAL)
        numSizer  = wx.GridBagSizer(hgap=5, vgap=5)
        
        self.displayTxt = wx.TextCtrl(self, wx.ID_ANY, "0", 
                                      size=(155,-1),
                                      style=wx.TE_RIGHT|wx.TE_READONLY)
        
        for eachLabel, eachSize, eachHandler, eachPos in self.buttonData():
            button = self.buildButton(eachLabel, eachSize, eachHandler)
            if eachPos:
                numSizer.Add(button, pos=eachPos, flag=wx.CENTER) 
            else:
                vBtnSizer.Add(button)
                
        masterBtnSizer.Add(numSizer, 0, wx.ALL, 5)
        masterBtnSizer.Add(vBtnSizer, 0, wx.ALL, 5)
        mainSizer.Add(self.displayTxt, 0, wx.ALL, 5)
        mainSizer.Add(masterBtnSizer)
        self.SetSizer(mainSizer)
        mainSizer.Fit(self.parent)
        
    def buttonData(self):
        size=(45, 45)
        return (("0", size, self.onButton, (4,1)), 
                ("1", size, self.onButton, (3,0)),
                ("2", size, self.onButton, (3,1)), 
                ("3", size, self.onButton, (3,2)),
                ("4", size, self.onButton, (2,0)), 
                ("5", size, self.onButton, (2,1)),
                ("6", size, self.onButton, (2,2)), 
                ("7", size, self.onButton, (1,0)),
                ("8", size, self.onButton, (1,1)), 
                ("9", size, self.onButton, (1,2)),
                ("/", size, self.onOperation, (0,0)), 
                ("*", size, self.onOperation, (0,1)),
                ("-", size, self.onOperation, (0,2)),
                ("+", (45,100), self.onOperation, None),
                ("Enter", (45,100), self.onCalculate, None))
    
    def buildButton(self, label, size, handler):
        """
        Builds a button and binds it to an event handler.
        Returns the button object
        """
        button = wx.Button(self, wx.ID_ANY, label, size=size)
        self.Bind(wx.EVT_BUTTON, handler, button)
        return button
        
    def onOperation(self, event):
        """
        Add an operator to the equation
        """
        print "onOperation handler fired"

    def onButton(self, event):
        """
        Keeps the display up to date
        """
        # Get the button object
        buttonObj = event.GetEventObject()
        # Get the label of the button object
        buttonLbl = buttonObj.GetLabel()     
        
        self.onNumber(buttonLbl)
        
    def onCalculate(self):
        """
        Calculate the total
        """
        print 'in onCalculate'

class PyCalcFrame(wx.Frame):
    def __init__(self):
        wx.Frame.__init__(self, parent=None, id=wx.ID_ANY, 
                          title="Calculator")
        panel = MainPanel(self)
                        
class PyCalc(wx.App):
    def __init__(self, redirect=False, filename=None):
        wx.App.__init__(self, redirect, filename)
        
    def OnInit(self):
        # create frame here
        frame = PyCalcFrame()
        frame.Show()
        return True
    
def main():
    app = PyCalc()
    app.MainLoop()
    
if __name__ == "__main__":
    main()
